%% GOAL: Find the singular value decomposition of a matrix by "hand"
A=[1 1 0;0 -1 0]
%% want to decompose A as A=PDQ'
%% P=2x2 orthogonal
%% Q=3x3 othogonal
%% D=2x3 "diagonal"
%% first, diagonalize AA'=PDQ'QD'P'=P(DD')P'
[P D1]=eigs(A*A');
%% then reorder so they are in descending order
[D1 order]=sort(diag(D1),'descend');
D1=diag(D1)
P=P(:,order)
%% you can check that A*A'=P*D1*P'
%% also, P*P'=eye(2), so P is orthogonal
%%
%% the diagonal entries of D are the square roots of D1
D=zeros(2,3);
D(1,1)=sqrt(D1(1,1));
D(2,2)=sqrt(D1(2,2));
D
%% now to find Q, note that P'A=DQ'
%% so QD'=A'P
%% this only determines the columns of Q corresponding to non-zero singular values
%% in this case there are 2 singular values, both of which are non-zero, so the first and second columns of Q are determined
%% they can be found by multiplying by the inverse of the corresponding part of D
Q=A'*P*inv(D(1:2,1:2))
%%
%% Q should be a 3x3, so we need to use Gram-Schmidt to find a 3rd column for Q
%% easy to see that [0 0 1]' works
%%
%% more generally, you can tell matlab to find an ONB for null space of Q'
Q=[Q orth(null(Q'))]

%% more complicated example
AA=[1 2 3 4 5 6; 3 4 5 3 4 5; 9 8 7 6 5 4]
[PP DD1]=eig(AA*AA');
[DD1 order]=sort(diag(DD1),'descend');
DD1=diag(DD1)
PP=PP(:,order)
DD=zeros(size(AA));
for i=1:min(size(AA))
	DD(i,i)=sqrt(DD1(i,i));
	if not( DD(i,i)==0)
		nonzerosv=i;
	end
end
QQ=AA'*PP*inv(DD(1:nonzerosv,1:nonzerosv));
QQ=[QQ orth(null(QQ'))]

